#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: davidworrall

===== READS us-states.csv (daily download from https://github.com/nytimes/covid-19-data/blob/master/us-states.csv)
===== CONVERTS contiguous daily listings in us-states.csv to time-series
===== WRITES to stateDeathsUS.csv and stateCasesUS.csv

The aim of this code is to extract data from fname = "us-states.csv"  # file data: date, state, fips, cases, deaths
cobvert it to Cases and Deaths time series CSVfiles us_stateDeaths.csv and us_stateCases.csv

NB:
FIPS = Federal Information Processing Standard FIP codes are five-digit codes that are assigned to each U.S. county. See:
https://medium.com/plotly/what-is-a-fips-code-county-level-charts-in-python-4eff383a4cf6
also note: http://ghdx.healthdata.org/

(FYI:
 The format of the OSofTruth file (deaths_global.csv) is: Province/State	Country/Region	Lat	    Long	1/22/20 (wed) etc...
                                                 col:          0               1        2       3       4      5 .....
The date data in deaths_global.csv is then pre-padded (covid19data.py) to being on the Sunday prior to 1/22/20)
 i.e three days are pre-pended: 1/19/20, 1/20/20, 1/21/20

Dates in the current file (us-states.csv) begin '2020-01-21','2020-01-22'
The first Case (i.e. +ve test) is on 20-01-21, however the 1st Death is some days later.

So, in order for the Deaths time-series .csv file emerging from the processing here,
 needs to being on same date as that in deaths_global.csv, namelty 1/22/20
    - First 4 cols to contain State, Country, Lat, Long, 1/20/20, 1/20/20 prepended to the death data
    - NB date data in deaths_global.csv
    - NNB date format in us-states.csv is 2020-01-21 which needs to be converted to 1/22/20, to allow conctention of these files
    - Case DATA is not processed by defult (see global SAVE_CASE_DATA = False) as it is currently outside the scope of this project


version -1; date: 2020 10 30
    - read "us-states.csv"

version 0.0 write state Data deaths from New York Times data                    # incremental daily death Nrs

v0.01 2020-1??-?? finished conversion from incremental data entry to time series for each state
v0.2 2021-01-14  - modifications to accommodate incorporation with OSofT (death_globals.csv)
modified to make it US_ specific


"""
global SAVE_CASE_DATA ; SAVE_CASE_DATA = False
global COLUMN_LABELS ; COLUMN_LABELS = False                                     # set to False iconcatenating the oputput file onto another

import csv
# ==============================WHERE THINGS ARE IN THE PROJECT FILES =========
# choose one: 
global ORANGE_STICK ; ORANGE_STICK  = False                                     # the USB stick
global MBP_288      ; MBP_288       = True                                      # the Fraunhofer MacBook
global AAA_DWOR     ; AAA_DWOR      = False                                     # The AAA/CCC MacBook
if ORANGE_STICK:
    rootDirectory  = '/volumes/DRWORANGE32/COVID-19/'
elif MBP_288:
    rootDirectory  = '/Users/wor/documents/RESEARCH projects/COVID-19/'
elif AAA_DWOR:
    rootDirectory  = '/Users/davidworrall/Documents/RESEARCH/COVID-19/'
else:
    print ('\n ======== ROOT DIRECTORY NOT FOUND FOR COVID-19 PROJECT ======\n')
    
directory = rootDirectory
websiteDirectory = rootDirectory + 'website/'                                    #  for website
imageDirectory  = rootDirectory + 'graphImages/deaths_global/'                   # for non-website images
dataDirectory   = rootDirectory + 'dataSets/'                                    # root directory  for data

fname     = "us-states.csv"      # file data: date, state, fips, cases, deaths

dateCol     = 0
stateCol    = 1
FIPcodeCol  = 2
nrCasesCol  = 3 
nrDeathsCol = 4
stateList_USA = []  
dateList_USA  = []
stateDataDeathsDict_USA = {}                                                        # for the incremental daily death Nrs
stateDataCasesDict_USA  = {}                                                        # for the incremental daily cases  Nrs
stateDict_USA = {}


with open(dataDirectory+fname, newline='') as csvfile:
    ipdata = csv.reader(csvfile, delimiter=',', quotechar='"')                  # read entire content of file
    for row in ipdata:                                                          # row defined by \n
        if row [dateCol] not in dateList_USA:                                   # add it to the dateList_USA
            dateList_USA += [row[dateCol]]                                      # compile a unique list of dates,auto tie sorted
#            dateList_USA += [row[dateCol].replace('-', '/')]                   # compile a unique list of dates,auto tie sorted
        stateNameKey = row[stateCol]                                            
        # print (stateNameKey)
        if stateNameKey in stateDict_USA.keys():                                # state data is yyyy-mm-dd, stateName, stateNr, nrCases, nrDeaths
            stateDict_USA[stateNameKey] += [row]                                # add state data to the state key for countyDict
        else:
            stateDict_USA[stateNameKey] = [row]                                 # create state_county key for countyDict

dateList_USA = dateList_USA[1:]                                                 # drop the column headers? we are producing a csv, so maybe no...!
del stateDict_USA['state']                                                      # delete the row heading entry, whose key is 'state'

datePad = ['2020-01-19', '2020-01-20']                                          # 2020-01-21 (the first date in the DB) is a Tues,so pre-pad for a Sunday start
                                                                                # NB keep this hyphen date format until writing 2 file
dateList_USA = datePad + dateList_USA                                           # concatenate!
# col/indx   0    1   2    3    4       5                                       # so dateData will beigin at [:4[]]
#          state,USA,lat,long,datepad1,datepad2
datePad = ''
#          ',USA,lat,long,22nd,'                                            # insert len (datePad days + nr day in datePad of 0 data)
dataPad =  ',USA,0,0,0,'                                            # insert len (datePad days + nr day in datePad of 0 data)

print ("nr dates: ", len (dateList_USA))

# ====== create death and cases dictionaries ==================================
for stateNameKey in stateDict_USA.keys():
    stateDataDeathsDict_USA[stateNameKey] = (len(dateList_USA)) * ['0']         # a list for DEATH numbers for each stateNameKey date
    stateDataCasesDict_USA [stateNameKey] = (len(dateList_USA)) * ['0']         # a list for CASE  numbers for each stateNameKey date
    for dateEntry in stateDict_USA[stateNameKey]:                               # necessary because the DB only has entries for events;ie doesn't includenot nulls
        indx = dateList_USA.index(dateEntry[dateCol])                           # the day number (i.e index of d (first) date of stateNameKey entry in the dateList_USA
#        print ("dateEntry:", dateEntry)                                        # each dateEnry is a list:[date, stateName,fips, #cases, #deaths]
# store the values in the stateNameKey entry in the stateDataDeathsDict_USA and stateDataCasesDict_USA
        if int(dateEntry[nrDeathsCol]) >0:                                           # if there is a death on that date 
                stateDataDeathsDict_USA[stateNameKey][indx] = dateEntry[nrDeathsCol] # insert the nr of deaths, else leave as 0
        if int(dateEntry[nrCasesCol]) >0:                                            # if there is a case on that date
                stateDataCasesDict_USA[stateNameKey] [indx] = dateEntry[nrCasesCol]  # insert the nr of cases, else leave as 0

with open(dataDirectory+'us_stateDeaths.csv', 'w', newline='\n') as csvfile:
    fieldNames = 'State,Country,Lat,Long,'                                      # NB: date list[0] (== 'date'`) already omitted
    tmp1 = dateList_USA[3:]                                                     # omit the dates not needed
    tmp2 = ','.join(tmp1)                                                       # make a csv string of states etc + dates 
    if COLUMN_LABELS:
        print ('COlumn Labels:\n"' +fieldNames+ tmp2 )
        csvfile.write(fieldNames + tmp2 + '\n')

    for dictKee in sorted(stateDataDeathsDict_USA.keys()):                      # alphabeticat order - why not?
        dataRowPad = ','.join(dictKee.split('_'))
        rowString = dataRowPad +dataPad+ ','.join (stateDataDeathsDict_USA[dictKee][4:])
#        rowString = dataRowPad + ','.join (stateDataDeathsDict_USA[dictKee])    # test to defer padding with OSofT
        csvfile.write(rowString+'\n')
        
if SAVE_CASE_DATA:        
    with open(dataDirectory+'us_stateCases.csv', 'w', newline='\n') as csvfile:
        fieldNames = 'State,County,Lat,Long,' +  ','.join(dateList_USA)         # NB: date`list[0] (== 'date'`) already omitted
        csvfile.write(fieldNames+'\n')
        for dictKee in sorted(stateDataCasesDict_USA.keys()):                   # alphabeticat order - why not?
            dataRowPad = ','.join(dictKee.split('_'))
            rowString = dataRowPad +dataPad+ ','.join (stateDataCasesDict_USA[dictKee])
            csvfile.write(rowString+'\n')

"""

with open(dataDirectory+'us_stateDeaths.csv', 'w', newline='\n') as csvfile:
    fieldNames = 'State,Country,Lat,Long,' + ','.join(dateList)                 # NB: date list[0] (== 'date'`) already omitted
    csvfile.write(fieldNames+'\n')
    for dictKee in sorted(stateDataDeathsDict.keys()):                          # alphabeticat order - why not?
        dataRowPad = ','.join(dictKee.split('_'))
        rowString = dataRowPad +dataPad+ ','.join (stateDataDeathsDict[dictKee])
        csvfile.write(rowString+'\n')
        

"""